/*  Copyright (C) 2004 Alexander Neundorf <neundorf@kde.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "qcppdialogimpl.h"

#include <qcombobox.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qtextbrowser.h>
#include <qlistbox.h>
#include <qlineedit.h>
#include <qfiledialog.h>
#include <qdir.h>
#include <qfile.h>
#include <qmessagebox.h>
#include <qsettings.h>
#include <qevent.h>
#include <qcstring.h>
#include <qprogressdialog.h>
#include <qapplication.h>
#include <qprocess.h>
#include <qfileinfo.h>
#include <qregexp.h>

#include <iostream.h>

#include <errno.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/ioctl.h>
#include <sys/termios.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/select.h>
#include <fcntl.h>

void millisleep(int ms)
{
   struct timeval tv;
   tv.tv_sec=0;
   tv.tv_usec=ms*1000;
   select(0, 0, 0, 0, &tv);
}

QCPPDialogImpl::QCPPDialogImpl(QWidget* parent)
   :CuteCommDlg(parent)
   ,m_isConnected(false)
   ,m_fd(-1)
   ,m_cmdBufIndex(0)
   ,m_notifier(0)
   ,m_sx(0)
   ,m_progress(0)
   ,m_progressStepSize(1000)
,m_fileDlg(0)
{
   connect(m_connectPb, SIGNAL(clicked()), this, SLOT(connectOrDisconnectTTY()));
   connect(m_clearOutputPb, SIGNAL(clicked()), m_outputView, SLOT(clear()));
   connect(m_clearInputPb, SIGNAL(clicked()), m_oldCmdsLb, SLOT(clear()));

   connect(m_cmdLe, SIGNAL(returnPressed()), this, SLOT(execCmd()));

   connect(m_sendPb, SIGNAL(clicked()), this, SLOT(sendFile()));
   connect(m_aboutPb, SIGNAL(clicked()), this, SLOT(showAboutMsg()));
   connect(m_quitPb, SIGNAL(clicked()), this, SLOT(close()));

   connect(m_oldCmdsLb, SIGNAL(clicked(QListBoxItem*)), this, SLOT(oldCmdClicked(QListBoxItem*)));
   connect(m_oldCmdsLb, SIGNAL(doubleClicked(QListBoxItem*)), this, SLOT(execCmd()));

   connect(m_connectPb, SIGNAL(clicked()), this, SLOT(saveSettings()));
//   connect(m_deviceCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_baudCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_dataBitsCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_parityCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_stopCb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_protoPb, SIGNAL(activated(int)), this, SLOT(saveSettings()));
   connect(m_softwareCb, SIGNAL(clicked()), this, SLOT(saveSettings()));
   connect(m_hardwareCb, SIGNAL(clicked()), this, SLOT(saveSettings()));

   m_outputView->setWrapPolicy(QTextEdit::Anywhere);
   m_outputView->setWordWrap(QTextEdit::WidgetWidth);

   readSettings();

   disconnectTTY();

   m_cmdLe->installEventFilter(this);
}

void QCPPDialogImpl::resizeEvent(QResizeEvent *e)
{
   QWidget::resizeEvent(e);
   saveSettings();
}

void QCPPDialogImpl::saveSettings()
{
   QSettings settings;
   settings.writeEntry("/cutecom/HardwareHandshake", m_hardwareCb->isChecked());
   settings.writeEntry("/cutecom/SoftwareHandshake", m_softwareCb->isChecked());
//   settings.writeEntry("/cutecom/Device", m_deviceCb->currentItem());
   settings.writeEntry("/cutecom/Baud", m_baudCb->currentItem());
   settings.writeEntry("/cutecom/Databits", m_dataBitsCb->currentItem());
   settings.writeEntry("/cutecom/Parity", m_parityCb->currentItem());
   settings.writeEntry("/cutecom/Stopbits", m_stopCb->currentItem());
   settings.writeEntry("/cutecom/Protocol", m_protoPb->currentItem());
   settings.writeEntry("/cutecom/width", width());
   settings.writeEntry("/cutecom/height", height());

   QString currentDevice=m_deviceCb->currentText();
   settings.writeEntry("/cutecom/CurrentDevice", currentDevice);
   bool currentDeviceIsInList=false;
   QStringList devices;
   for (int i=0; i<m_deviceCb->count(); i++)
   {
      QString s=m_deviceCb->text(i);
      devices<<s;
      if (s==currentDevice)
         currentDeviceIsInList=true;
   }

   if (!currentDeviceIsInList)
      devices<<currentDevice;

   settings.writeEntry("/cutecom/AllDevices", devices);

   int historyCount=m_oldCmdsLb->count();
   if (historyCount>20)
      historyCount=20;

   QStringList saveHist;
   for (int i=m_oldCmdsLb->count()-historyCount; i<m_oldCmdsLb->count(); i++)
      saveHist<< m_oldCmdsLb->text(i);
   settings.writeEntry("/cutecom/History", saveHist);
}

void QCPPDialogImpl::readSettings()
{
   QSettings settings;
   m_hardwareCb->setChecked(settings.readBoolEntry("/cutecom/HardwareHandshake", false));
   m_softwareCb->setChecked(settings.readBoolEntry("/cutecom/SoftwareHandshake", false));
//   m_deviceCb->setCurrentItem(settings.readNumEntry("/cutecom/Device", 0));
   m_baudCb->setCurrentItem(settings.readNumEntry("/cutecom/Baud", 7));
   m_dataBitsCb->setCurrentItem(settings.readNumEntry("/cutecom/Databits", 3));
   m_parityCb->setCurrentItem(settings.readNumEntry("/cutecom/Parity", 0));
   m_stopCb->setCurrentItem(settings.readNumEntry("/cutecom/Stopbits", 0));
   m_protoPb->setCurrentItem(settings.readNumEntry("/cutecom/Protocol", 0));

   int x=settings.readNumEntry("/cutecom/width", -1);
   int y=settings.readNumEntry("/cutecom/height", -1);
   if ((x>100) && (y>100))
      resize(x,y);

   bool entryFound=false;
   QStringList devices=settings.readListEntry("/cutecom/AllDevices", &entryFound);
   if (!entryFound)
      devices<<"/dev/ttyS0"<<"/dev/ttyS1"<<"/dev/ttyS2"<<"/dev/ttyS3";

   m_deviceCb->insertStringList(devices);

   m_deviceCb->setCurrentText(settings.readEntry("/cutecom/CurrentDevice", "/dev/ttyS0"));

   QStringList history=settings.readListEntry("/cutecom/History");
   m_oldCmdsLb->insertStringList(history);

}

void QCPPDialogImpl::showAboutMsg()
{
   QMessageBox::about(this, tr("About CuteCom"), tr("This is CuteCom 0.0.8<br>(c)2004 Alexander Neundorf, &lt;neundorf@kde.org&gt;<br>Licensed under the GNU GPL v2"));
}

void QCPPDialogImpl::sendFile()
{
   if (m_fileDlg==0)
   {
      m_fileDlg=new QFileDialog(QDir::homeDirPath());
      m_fileDlg->setMode(QFileDialog::ExistingFile);
   }

   QString filename;
   if ( m_fileDlg->exec() == QDialog::Accepted )
      filename = m_fileDlg->selectedFile();
   else
      return;

   if (m_protoPb->currentText()=="Script")
   {
      QFile file(filename);
      if (!file.open(IO_ReadOnly))
      {
         QMessageBox::information(this, tr("Opening file failed"), tr("Could not open file %1").arg(filename));
         return;
      }

      QTextStream stream(&file);
      while (!stream.atEnd())
      {
         QString nextLine=stream.readLine();
         nextLine=nextLine.left((unsigned int)nextLine.find('#'));
         if (nextLine.isEmpty())
            continue;

         if (!sendString(nextLine))
         {
            QMessageBox::information(this, tr("Comm error"), tr("Sending failed"));
            return;
         }
      }
   }
   else if (m_protoPb->currentText()=="Plain")
   {
      QFile file(filename);
      if (!file.open(IO_ReadOnly))
      {
         QMessageBox::information(this, tr("Opening file failed"), tr("Could not open file %1").arg(filename));
         return;
      }
      QByteArray data=file.readAll();
      delete m_progress;
      m_progress=new QProgressDialog(tr("Sending file..."), tr("Cancel"), 100, this, "progress", TRUE);
      m_progress->setMinimumDuration(100);
      unsigned int step=data.size()/100;
      for (unsigned int i=0; i<data.size(); i++)
      {
         if ((i%step)==0)
         {
            m_progress->setProgress(i/step);
            qApp->processEvents();
         }
         sendByte(data.data()[i]);
//         if (!sendByte(data.data()[i]))
         if (0)
         {
            QMessageBox::information(this, tr("Comm error"), tr("Sending failed (%1/%2").arg(i).arg(data.count()));
            break;
         }
         if ( m_progress->wasCancelled() )
            break;
      }
      delete m_progress;
      m_progress=0;
   }
   else if ((m_protoPb->currentText()=="XModem")
            || (m_protoPb->currentText()=="YModem")
            || (m_protoPb->currentText()=="ZModem"))
   {
//      QProcess sx(this);
      disconnectTTY();
      m_sx=new QProcess(this);
      m_sx->addArgument("sh");
      m_sx->addArgument("-c");
//      QString tmp=QString("sx -vv \"")+filename+"\" < "+m_deviceCb->currentText()+" > "+m_deviceCb->currentText();
      QString tmp=QString("sz ");
      if (m_protoPb->currentText()=="XModem")
         tmp+="--xmodem ";
      else if (m_protoPb->currentText()=="YModem")
         tmp+="--ymodem ";
      else if (m_protoPb->currentText()=="ZModem")
         tmp+="--zmodem ";
      tmp=tmp+"-vv \""+filename+"\" < "+m_deviceCb->currentText()+" > "+m_deviceCb->currentText();
      m_sx->addArgument(tmp);
      m_sx->setCommunication(QProcess::Stderr);

      connect(m_sx, SIGNAL(readyReadStderr()), this, SLOT(readFromStderr()));
/*      m_sx->addArgument("sx");
      m_sx->addArgument("-vv");
      m_sx->addArgument(filename);
      m_sx->setCommunication(QProcess::Stdin|QProcess::Stdout|QProcess::Stderr);

      connect(m_sx, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()));
      connect(m_sx, SIGNAL(readyReadStderr()), this, SLOT(readFromStderr()));*/
      connect(m_sx, SIGNAL(processExited()), this, SLOT(sendDone()));
      if (!m_sx->start())
      {
         QMessageBox::information(this, tr("Comm error"), tr("Could not start sx"));
         delete m_sx;
         m_sx=0;
         connectTTY();
         return;
      }
      m_progress=new QProgressDialog(tr("Sending file via xmodem..."), tr("Cancel"), 100, this, "progress", TRUE);
      m_progress->setMinimumDuration(100);
      QFileInfo fi(filename);
      m_progressStepSize=fi.size()/1024/100;
//    cerr<<"while(isRunning)"<<endl;
      m_progress->setProgress(0);
      while (m_sx->isRunning())
      {
         qApp->processEvents();
         millisleep(10);
      }
      cerr<<"----------------- sx done"<<endl;

      delete m_sx;
      m_sx=0;
      delete m_progress;
      m_progress=0;
      connectTTY();
   }
   else
   {
      QMessageBox::information(this, tr("Opening file failed"), tr("Protocol %1 not supported yet").arg(m_protoPb->currentText()));
   }
}

void QCPPDialogImpl::readFromStdout()
{
   QByteArray ba=m_sx->readStdout();
   cerr<<"readFromStdout() "<<ba.count()<<endl;
   unsigned int bytesToWrite=ba.count();
   char* src=ba.data();
   while (bytesToWrite>0)
   {
      int bytesWritten=::write(m_fd, src, (bytesToWrite>CUTECOMM_BUFSIZE?CUTECOMM_BUFSIZE:bytesToWrite));
      if (bytesWritten<0)
      {
         cerr<<"readFromStdout() error "<<bytesWritten<<" , "<<bytesToWrite<<" left"<<endl;
         return;
      }
      src+=bytesWritten;
      bytesToWrite-=bytesWritten;

   }
}

void QCPPDialogImpl::readFromStderr()
{
   QByteArray ba=m_sx->readStderr();
   cerr<<"readFromStderr() "<<ba.count()<<endl;
   if (m_progress==0)
      return;
   QString s(ba);
   QRegExp regex(".+\\d+/ *(\\d+)k.*");
   int pos=regex.search(s);
   if (pos>-1)
   {
      QString captured=regex.cap(1);
      cerr<<"captured kb: -"<<captured.latin1()<<"-"<<endl;
      int kb=captured.toUInt();
      if ((kb%m_progressStepSize)==0)
      {
         int p=kb/m_progressStepSize;
         if (p<100)
            m_progress->setProgress(p);
      }
   }
   else
      cerr<<"--------"<<s.latin1()<<"-"<<endl;
/*   for (unsigned int i=0; i<ba.count(); i++)
   {
      char c=ba.data()[i];
      unsigned int tmp=c;
      if (isprint(tmp))
         cerr<<c;
      else
         fprintf(stderr, " \\0x%02x ", tmp&0xff);
   }*/
}

void QCPPDialogImpl::sendDone()
{
   cerr<<"sx exited"<<endl;
}

bool QCPPDialogImpl::eventFilter(QObject* watched, QEvent *e)
{
   QKeyEvent *ke=(QKeyEvent*)e;
   if ((watched==m_cmdLe)
       && (e->type()==QEvent::KeyPress)
       && (ke->state()==0))
   {
      if (ke->key()==Key_Up)
      {
         prevCmd();
         return true;
      }
      else if (ke->key()==Key_Down)
      {
         nextCmd();
         return true;
      }
   }
   return false;
}

void QCPPDialogImpl::oldCmdClicked(QListBoxItem* item)
{
   if (item==0)
      return;
   int index=m_oldCmdsLb->index(item);
   m_cmdLe->setText(item->text());
   m_cmdBufIndex=m_oldCmdsLb->count()-index;
   m_cmdLe->setFocus();
}

void QCPPDialogImpl::prevCmd()
{
   if (m_oldCmdsLb->count()<=m_cmdBufIndex)
      return;
//   cerr<<"prevCmd() count: "<<m_oldCmdsLb->count()<<" bufIndex: "<<
   m_cmdBufIndex++;
   m_oldCmdsLb->setCurrentItem(m_oldCmdsLb->count()-m_cmdBufIndex);
   m_oldCmdsLb->setSelected(m_oldCmdsLb->currentItem(), true);
   m_cmdLe->setText(m_oldCmdsLb->currentText());

//   cerr<<"prev() count: "<<m_oldCmdsLb->count()<<" bufIndex: "<<m_cmdBufIndex<<endl;
}

void QCPPDialogImpl::nextCmd()
{
   if (m_cmdBufIndex==0)
      return;
   m_cmdBufIndex--;
   if (m_cmdBufIndex==0)
   {
      m_cmdLe->clear();
      m_oldCmdsLb->setSelected(m_oldCmdsLb->currentItem(), false);
   }
   else
   {
      m_oldCmdsLb->setCurrentItem(m_oldCmdsLb->count()-m_cmdBufIndex);
      m_cmdLe->setText(m_oldCmdsLb->currentText());
   }
//   cerr<<"next() count: "<<m_oldCmdsLb->count()<<" bufIndex: "<<m_cmdBufIndex<<endl;
}

void QCPPDialogImpl::execCmd()
{
   m_cmdBufIndex=0;
   QString cmd=m_cmdLe->text().stripWhiteSpace();
   m_cmdLe->clear();
   if (!cmd.isEmpty())
   {
      if ((m_oldCmdsLb->count()<1) || (m_oldCmdsLb->text(m_oldCmdsLb->count()-1)!=cmd))
      {
         m_oldCmdsLb->insertItem(cmd);
         m_oldCmdsLb->setCurrentItem(m_oldCmdsLb->count()-1);
//       m_oldCmdsLb->setSelected(m_oldCmdsLb->currentItem(), false);
         if (m_oldCmdsLb->count()>50)
            m_oldCmdsLb->removeItem(0);
         saveSettings();
      }
   }
   m_oldCmdsLb->clearSelection();
   if (m_fd==-1)
      return;

   sendString(cmd);

/*   cerr<<"paras: "<<m_outputView->paragraphs()<<endl;
   if (m_outputView->paragraphs()>1100)
   {
      m_outputView->setUpdatesEnabled(false);
      m_outputView->setSelection(0, 0, 100, 0);
      m_outputView->removeSelectedText();
      m_outputView->scrollToBottom();
      m_outputView->append("abc\n");
      m_outputView->setUpdatesEnabled(true);
   }
   else*/
/*   m_outputView->setCursorPosition(m_outputView->paragraphs()-1, 0);
   static int i=0;
   i++;
   QString s=QString::number(i);
   s=" -"+s+"-\n ";
   m_outputView->insert(s);*/
/*   m_outputView->insert("abc");
   m_outputView->insert("def");
   m_outputView->insert("ghi\n");
   m_outputView->update();*/
}

bool QCPPDialogImpl::sendString(const QString& s)
{
   const char *bytes=s.latin1();
   for (unsigned int i=0; i<s.length(); i++)
   {
      if (!sendByte(*bytes))
         return false;
      bytes++;
      millisleep(1);
   }
   if (!sendByte('\n'))
      return false;
   return true;
}

bool QCPPDialogImpl::sendByte(char c)
{
   if (m_fd==-1)
      return false;
   int res=::write(m_fd, &c, 1);
   if (res<1)
   {
      cerr<<"write returned "<<res<<" errno: "<<errno<<endl;
      perror("write\n");
      return false;
   }
//   else
//      cerr<<" \""<<c<<"\"; ";
   return true;
}

void QCPPDialogImpl::connectOrDisconnectTTY()
{
   if (m_fd==-1)
      connectTTY();
   else
      disconnectTTY();
}

void QCPPDialogImpl::connectTTY()
{
   QString dev=m_deviceCb->currentText();
   int baudrate=m_baudCb->currentText().toInt();
   int dataBits=m_dataBitsCb->currentText().toInt();
   QString parity=m_parityCb->currentText();
   QString stop=m_stopCb->currentText();
   bool softwareHandshake=m_softwareCb->isChecked();
   bool hardwareHandshake=m_hardwareCb->isChecked();

//  m_fd=open(dev.latin1(), O_RDWR | O_NOCTTY | O_NONBLOCK);
// m_fd=open(dev.latin1(), O_RDWR | O_NOCTTY);
   m_fd=open(dev.latin1(), O_RDWR | O_NDELAY);
   if (m_fd<0)
   {
      cerr<<"opening failed"<<endl;
      m_fd=-1;
      QMessageBox::information(this, tr("Error"), tr("Could not open %1").arg(dev));
      return;
   }
   int n = fcntl(m_fd, F_GETFL, 0);
   fcntl(m_fd, F_SETFL, n & ~O_NDELAY);
/*   {
      unsigned int i;
      struct termios tty;
      char *c =(char*)&tty;
      if (tcgetattr(m_fd, &tty)!=0)
         cerr<<"tcgetattr() 1 failed"<<endl;
      fprintf(stderr,"#################\n");
      for (i=0; i<sizeof(tty); i++)
      {
         unsigned int t=*c;
         if (i%8 == 0)
            fprintf(stderr,"\n");
         fprintf(stderr, " 0x%02x", t&0xff);
         c++;
      }
   }*/

   if (tcgetattr(m_fd, &m_oldtio)!=0)
        cerr<<"tcgetattr() 2 failed"<<endl;

   setNewOptions(baudrate, dataBits, parity, stop, softwareHandshake, hardwareHandshake);

   m_connectPb->setText(tr("Close device"));
   m_deviceCb->setEnabled(false);
   m_baudCb->setEnabled(false);
   m_dataBitsCb->setEnabled(false);
   m_parityCb->setEnabled(false);
   m_stopCb->setEnabled(false);
   m_softwareCb->setEnabled(false);
   m_hardwareCb->setEnabled(false);
   m_isConnected=true;


   m_notifier=new QSocketNotifier(m_fd, QSocketNotifier::Read, this);
   connect(m_notifier, SIGNAL(activated(int)), this, SLOT(readData(int)));
   m_outputView->setEnabled(true);
   m_oldCmdsLb->setEnabled(true);
   m_cmdLe->setEnabled(true);
   m_sendPb->setEnabled(true);
   m_protoPb->setEnabled(true);

   m_cmdLe->setFocus();
}

void QCPPDialogImpl::disconnectTTY()
{
   cerr<<"closing "<<m_fd<<endl;
   if (m_fd!=-1)
   {
      tcsetattr(m_fd, TCSANOW, &m_oldtio);
      ::close(m_fd);
   }
   m_fd=-1;
   m_connectPb->setText(tr("Open device"));
   m_deviceCb->setEnabled(true);
   m_baudCb->setEnabled(true);
   m_dataBitsCb->setEnabled(true);
   m_parityCb->setEnabled(true);
   m_stopCb->setEnabled(true);
   m_softwareCb->setEnabled(true);
   m_hardwareCb->setEnabled(true);

   m_outputView->setEnabled(false);
   m_oldCmdsLb->setEnabled(false);
   m_cmdLe->setEnabled(false);
   m_sendPb->setEnabled(false);
   m_protoPb->setEnabled(false);

   m_connectPb->setFocus();

   m_isConnected=false;
   delete m_notifier;
   m_notifier=0;
}

/** This function features some code from minicom 2.0.0, src/sysdep1.c */
void QCPPDialogImpl::setNewOptions(int baudrate, int databits, const QString& parity, const QString& stop, bool softwareHandshake, bool hardwareHandshake)
{
   struct termios newtio;
//   memset(&newtio, 0, sizeof(newtio));
   if (tcgetattr(m_fd, &newtio)!=0)
   cerr<<"tcgetattr() 3 failed"<<endl;

   /*{
      unsigned int i;
      char *c =(char*)&newtio;
      fprintf(stderr,"*****************\n");
      for (i=0; i<sizeof(newtio); i++)
      {
         unsigned int t=*c;
         if (i%8 == 0)
            fprintf(stderr,"\n");
         fprintf(stderr, " 0x%02x", t&0xff);
         c++;
      }
   }*/


   speed_t _baud=0;
   switch (baudrate)
   {
   case 600:
      _baud=B600;
      break;
   case 1200:
      _baud=B1200;
      break;
   case 2400:
      _baud=B2400;
      break;
   case 4800:
      _baud=B4800;
      break;
   case 9600:
      _baud=B9600;
      break;
//   case 14400:
//      _baud=B14400;
//      break;
   case 19200:
      _baud=B19200;
      break;
//   case 28800:
//      _baud=B28800;
//      break;
   case 38400:
      _baud=B38400;
      break;
//   case 56000:
//      _baud=B56000;
//      break;
   case 57600:
      _baud=B57600;
      break;
   case 115200:
      _baud=B115200;
      break;
   case 230400:
      _baud=B230400;
      break;
   case 460800:
      _baud=B460800;
      break;
   case 576000:
      _baud=B576000;
      break;
   case 921600:
      _baud=B921600;
      break;
//   case 128000:
//      _baud=B128000;
//      break;
   default:
//   case 256000:
//      _baud=B256000;
      break;
   }
   cfsetospeed(&newtio, (speed_t)_baud);
   cfsetispeed(&newtio, (speed_t)_baud);

   /* We generate mark and space parity ourself. */
   if (databits == 7 && (parity=="Mark" || parity == "Space"))
      databits = 8;
   switch (databits)
   {
   case 5:
      newtio.c_cflag = (newtio.c_cflag & ~CSIZE) | CS5;
      break;
   case 6:
      newtio.c_cflag = (newtio.c_cflag & ~CSIZE) | CS6;
      break;
   case 7:
      newtio.c_cflag = (newtio.c_cflag & ~CSIZE) | CS7;
      break;
   case 8:
   default:
      newtio.c_cflag = (newtio.c_cflag & ~CSIZE) | CS8;
      break;
   }
   newtio.c_cflag |= CLOCAL | CREAD;

   //parity
   newtio.c_cflag &= ~(PARENB | PARODD);
   if (parity == "Even")
      newtio.c_cflag |= PARENB;
   else if (parity== "Odd")
      newtio.c_cflag |= (PARENB | PARODD);

   //hardware handshake
/*   if (hardwareHandshake)
      newtio.c_cflag |= CRTSCTS;
   else
      newtio.c_cflag &= ~CRTSCTS;*/
   newtio.c_cflag &= ~CRTSCTS;

   //stopbits
   if (stop=="2")
      newtio.c_cflag |= CSTOPB;
   else
      newtio.c_cflag &= ~CSTOPB;

//   newtio.c_iflag=IGNPAR | IGNBRK;
    newtio.c_iflag=IGNBRK;
//   newtio.c_iflag=IGNPAR;

   //software handshake
   if (softwareHandshake)
      newtio.c_iflag |= IXON | IXOFF;
   else
      newtio.c_iflag &= ~(IXON|IXOFF|IXANY);

   newtio.c_lflag=0;

   newtio.c_oflag=0;


   newtio.c_cc[VTIME]=2;
   newtio.c_cc[VMIN]=40;

//   tcflush(m_fd, TCIFLUSH);
   if (tcsetattr(m_fd, TCSANOW, &newtio)!=0)
      cerr<<"tcsetattr() 1 failed"<<endl;

   int mcs=0;
//   ioctl(m_fd, TIOCMODG, &mcs);
   ioctl(m_fd, TIOCMGET, &mcs);
   mcs |= TIOCM_RTS;
   ioctl(m_fd, TIOCMSET, &mcs);

   if (tcgetattr(m_fd, &newtio)!=0)
      cerr<<"tcgetattr() 4 failed"<<endl;
   //hardware handshake
   if (hardwareHandshake)
      newtio.c_cflag |= CRTSCTS;
   else
      newtio.c_cflag &= ~CRTSCTS;
/*  if (on)
     newtio.c_cflag |= CRTSCTS;
  else
     newtio.c_cflag &= ~CRTSCTS;*/
   if (tcsetattr(m_fd, TCSANOW, &newtio)!=0)
      cerr<<"tcsetattr() 2 failed"<<endl;

   /*{
      unsigned int i;
      char *c =(char*)&newtio;
      fprintf(stderr,"-----------------\n");
      tcgetattr(m_fd, &newtio);
      for (i=0; i<sizeof(newtio); i++)
      {
         unsigned int t=*c;
         if (i%8 == 0)
            fprintf(stderr,"\n");
         fprintf(stderr, " 0x%02x", t&0xff);
         c++;
      }
   }*/
}

void QCPPDialogImpl::readData(int fd)
{
   if (fd!=m_fd)
      return;
   int bytesRead=::read(m_fd, m_buf, CUTECOMM_BUFSIZE);

   if (bytesRead<0)
   {
      cerr<<"read result: "<<bytesRead<<endl;
      perror("read: \n");
      return;
   }

   const char* c=m_buf;
   if (m_sx!=0)
   {
      cerr<<"readData() "<<bytesRead<<endl;
      QByteArray ba;
      ba.duplicate(m_buf, bytesRead);
//      ba.setRawData(m_buf, bytesRead);
      m_sx->writeToStdin(ba);
//      ba.resetRawData(m_buf, bytesRead);
      return;
   }

   QString text;
   for (int i=0; i<bytesRead; i++)
   {
      if ((isprint(*c)) || (*c=='\n') || (*c=='\r'))
      {
         if (*c!='\r')
            text+=(*c);
      }
      else
      {
         char buf[16];
         unsigned int i=*c;
         snprintf(buf, 16, "\\0x%02x", i & 0xff);
         text+=buf;
      }
      c++;
   }

   if (m_outputView->paragraphs()>1100)
   {
      m_outputView->setUpdatesEnabled(false);
      m_outputView->setSelection(0, 0, 100, 0);
      m_outputView->removeSelectedText();
      m_outputView->scrollToBottom();
      m_outputView->setCursorPosition(m_outputView->paragraphs()-1, m_outputView->paragraphLength(m_outputView->paragraphs()-1));
      m_outputView->insert(text);
      m_outputView->setUpdatesEnabled(true);
   }
   else
   {
      m_outputView->setCursorPosition(m_outputView->paragraphs()-1, m_outputView->paragraphLength(m_outputView->paragraphs()-1));
      m_outputView->insert(text);
   }
}


